from unittest import TestCase
from unittest.mock import patch

from embed_video.backends import UnknownIdException, YoutubeBackend, detect_backend


class YoutubeBackendTestCase(TestCase):
    urls = (
        ("http://youtu.be/jsrRJyHBvzw", "jsrRJyHBvzw"),
        ("http://youtu.be/n17B_uFF4cA", "n17B_uFF4cA"),
        ("http://youtu.be/t-ZRX8984sc", "t-ZRX8984sc"),
        ("https://youtu.be/t-ZRX8984sc", "t-ZRX8984sc"),
        ("http://youtube.com/watch?v=jsrRJyHBvzw", "jsrRJyHBvzw"),
        ("https://youtube.com/watch?v=jsrRJyHBvzw", "jsrRJyHBvzw"),
        ("http://www.youtube.com/v/0zM3nApSvMg?rel=0", "0zM3nApSvMg"),
        ("https://www.youtube.com/v/0zM3nApSvMg?rel=0", "0zM3nApSvMg"),
        ("http://www.youtube.com/embed/0zM3nApSvMg?rel=0", "0zM3nApSvMg"),
        ("https://www.youtube.com/embed/0zM3nApSvMg?rel=0", "0zM3nApSvMg"),
        ("http://www.youtube.com/watch?v=jsrRJyHBvzw", "jsrRJyHBvzw"),
        ("https://www.youtube.com/watch?v=t-ZRX8984sc", "t-ZRX8984sc"),
        ("http://www.youtube.com/watch?v=iwGFalTRHDA&feature=related", "iwGFalTRHDA"),
        ("https://www.youtube.com/watch?v=iwGFalTRHDA&feature=related", "iwGFalTRHDA"),
        (
            "http://www.youtube.com/watch?feature=player_embedded&v=2NpZbaAIXag",
            "2NpZbaAIXag",
        ),
        (
            "https://www.youtube.com/watch?feature=player_embedded&v=2NpZbaAIXag",
            "2NpZbaAIXag",
        ),
        (
            "https://www.youtube.com/watch?v=XPk521voaOE&feature=youtube_gdata_player",
            "XPk521voaOE",
        ),
        (
            "http://www.youtube.com/watch?v=6xu00J3-g2s&list=PLb5n6wzDlPakFKvJ69rJ9AJW24Aaaki2z",
            "6xu00J3-g2s",
        ),
        ("https://m.youtube.com/#/watch?v=IAooXLAPoBQ", "IAooXLAPoBQ"),
        ("https://m.youtube.com/watch?v=IAooXLAPoBQ", "IAooXLAPoBQ"),
        ("https://www.youtube.com/shorts/cOoQ7pc0CoY", "cOoQ7pc0CoY"),
        ("https://youtube.com/shorts/cOoQ7pc0CoY", "cOoQ7pc0CoY"),
    )

    instance = YoutubeBackend

    def test_detect(self):
        for url in self.urls:
            backend = detect_backend(url[0])
            self.assertIsInstance(backend, self.instance)

    def test_code(self):
        for url in self.urls:
            backend = self.instance(url[0])
            self.assertEqual(backend.code, url[1])

    def test_youtube_keyerror(self):
        """Test for issue #7"""
        backend = self.instance("http://youtube.com/watch?id=5")
        self.assertRaises(UnknownIdException, backend.get_code)

    def test_thumbnail(self):
        for url in self.urls:
            backend = self.instance(url[0])
            self.assertIn(url[1], backend.thumbnail)

    def test_get_better_resolution_youtube(self):
        backend = self.instance("https://www.youtube.com/watch?v=1Zo0-sWD7xE")
        self.assertIn(
            "img.youtube.com/vi/1Zo0-sWD7xE/maxresdefault.jpg", backend.thumbnail
        )

    @patch("embed_video.backends.EMBED_VIDEO_YOUTUBE_CHECK_THUMBNAIL", False)
    def test_youtube_not_check_thumbnail(self):
        backend = self.instance("https://www.youtube.com/watch?v=not-exist")
        self.assertIn("img.youtube.com/vi/not-exist/hqdefault.jpg", backend.thumbnail)



from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.utils.safestring import mark_safe

from embed_video.backends import (
    UnknownBackendException,
    VideoDoesntExistException,
    detect_backend,
)
from embed_video.fields import EmbedVideoField


class AdminVideoWidget(forms.TextInput):
    """
    Widget for video input in administration. If empty it works just like
    :py:class:`django.forms.TextInput`. Otherwise it renders embedded video
    together with input field.

    .. todo::

        Django 1.6 provides better parent for this widget -
        :py:class:`django.forms.URLInput`.

    """

    output_format = (
        '<div style="float:left" class="video">'
        "{video}<br />{input}</div>"
        '<hr style="visibility: hidden; clear:both">'
    )

    def __init__(self, attrs=None):
        """
        :type attrs: dict
        """
        default_attrs = {"size": "40"}
        self.validator = URLValidator()

        if attrs:
            default_attrs.update(attrs)

        super().__init__(default_attrs)

    def render(self, name, value="", attrs=None, size=(420, 315), renderer=None):
        """
        :type name: str
        :type attrs: dict
        """

        output = super().render(name, value, attrs, renderer)

        if not value:
            return output

        try:
            self.validator(value)
            backend = detect_backend(value)
            return mark_safe(
                self.output_format.format(
                    video=backend.get_embed_code(*size), input=output
                )
            )
        except (UnknownBackendException, ValidationError, VideoDoesntExistException):
            return output


class AdminVideoMixin:
    """
    Mixin using :py:class:`AdminVideoWidget` for fields with
    :py:class:`~embed_video.fields.EmbedVideoField`.

    Usage::

        from django.contrib import admin
        from embed_video.admin import AdminVideoMixin
        from .models import MyModel

        class MyModelAdmin(AdminVideoMixin, admin.ModelAdmin):
            pass

        admin.site.register(MyModel, MyModelAdmin)

    """

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        :type db_field: str
        """
        if isinstance(db_field, EmbedVideoField):
            return db_field.formfield(widget=AdminVideoWidget)

        return super().formfield_for_dbfield(db_field, **kwargs)



import re
import urllib.parse as urlparse
from json import dumps
from unittest import TestCase
from unittest.mock import Mock, patch

import requests_mock
from django.http import HttpRequest
from django.template import TemplateSyntaxError
from django.template.base import Template
from django.template.context import RequestContext
from django.test.client import RequestFactory

from embed_video.templatetags.embed_video_tags import VideoNode

URL_PATTERN = re.compile(r'src="?\'?([^"\'>]*)"')


class EmbedTestCase(TestCase):
    def render_template(self, template_string, context=None):
        response = RequestContext(HttpRequest(), context)
        return Template(template_string).render(response).strip()

    def assertRenderedTemplate(self, template_string, output, context=None):
        rendered_output = self.render_template(template_string, context=context)
        self.assertEqual(rendered_output, output.strip())

    def url_dict(self, url):
        """
        Parse the URL into a format suitable for comparison, ignoring the query
        parameter order.
        """

        parsed = urlparse.urlparse(url)
        query = urlparse.parse_qs(parsed.query)

        return {
            "scheme": parsed.scheme,
            "netloc": parsed.netloc,
            "path": parsed.path,
            "params": parsed.params,
            "query": query,
            "fragment": parsed.fragment,
        }

    def assertUrlEqual(self, actual, expected, msg=None):
        """Assert two URLs are equal, ignoring the query parameter order."""
        actual_dict = self.url_dict(actual)
        expected_dict = self.url_dict(expected)

        self.assertEqual(actual_dict, expected_dict, msg=msg)

    def test_embed(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {% video ytb 'large' %}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="960" height="720" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_embed_invalid_url(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/edit?abcd=efgh' as ytb %}
                {{ ytb.url }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(template, "")

    def test_embed_with_none_instance(self):
        template = """
            {% with None as my_video %}
            {% load embed_video_tags %}
            {% video my_video %}{% endwith %}
        """
        self.assertRenderedTemplate(template, "")

    def test_embed_empty_string(self):
        template = """
            {% load embed_video_tags %}
            {% video '' 'large' %}
        """
        self.assertRenderedTemplate(template, "")

    def test_direct_embed_tag(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://www.youtube.com/watch?v=jsrRJyHBvzw" "large" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="960" height="720" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_direct_embed_tag_with_default_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://www.youtube.com/watch?v=jsrRJyHBvzw" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="480" height="360" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_direct_embed_invalid_url(self):
        template = """
            {% load embed_video_tags %}
            {% video "https://soundcloud.com/xyz/foo" %}
        """
        self.assertRenderedTemplate(template, "")

    def test_user_size(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {% video ytb '800x800' %}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="800" height="800" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_wrong_size(self):
        template = Template(
            """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' 'so x huge' %}
        """
        )
        request = RequestContext(HttpRequest())
        self.assertRaises(TemplateSyntaxError, template.render, request)

    def test_tag_youtube(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {{ ytb.url }} {{ ytb.backend }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            "https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque " "YoutubeBackend",
        )

    def test_tag_vimeo(self):
        template = """
            {% load embed_video_tags %}
            {% video 'https://vimeo.com/72304002' as vimeo %}
                {{ vimeo.url }} {{ vimeo.backend }} {{ vimeo.info.duration }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template, "https://player.vimeo.com/video/72304002 VimeoBackend 176"
        )

    def test_tag_soundcloud(self):
        template = """
            {% load embed_video_tags %}
            {% video 'https://soundcloud.com/community/soundcloud-case-study-wildlife' as soundcloud %}
                {{ soundcloud.url }} {{ soundcloud.backend }}
            {% endvideo %}
        """

        # Soundcloud backend generates the following embed URL and fetches the payload from it
        # you can check the URL contents and update it as needed
        url = "https://soundcloud.com/oembed?format=json&url=https%3A%2F%2Fsoundcloud.com%2Fcommunity%2Fsoundcloud-case-study-wildlife"
        response = dumps(
            {
                "version": 1.0,
                "type": "rich",
                "provider_name": "SoundCloud",
                "provider_url": "https://soundcloud.com",
                "height": 400,
                "width": "100%",
                "title": "SoundCloud Case Study: Wildlife Control by SoundCloud Community",
                "description": "Listen to how Wildlife Control makes the most of the SoundCloud platform, and it's API.",
                "thumbnail_url": "https://i1.sndcdn.com/artworks-000042390403-ouou1g-t500x500.jpg",
                "html": "<iframe "
                'width="100%" height="400" scrolling="no" frameborder="no" '
                'src="https://w.soundcloud.com/player/?visual=true&url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F82244706&show_artwork=true"'
                "></iframe>",
                "author_name": "SoundCloud Community",
                "author_url": "https://soundcloud.com/community",
            }
        )

        with requests_mock.Mocker() as m:
            m.get(url, text=response)
            self.assertRenderedTemplate(
                template,
                "https://w.soundcloud.com/player/?visual=true&amp;url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F82244706&amp;show_artwork=true "
                "SoundCloudBackend",
            )

    @patch("embed_video.backends.EMBED_VIDEO_TIMEOUT", 0.000001)
    @patch("urllib3.connectionpool.log")
    @patch("embed_video.templatetags.embed_video_tags.logger")
    def test_empty_if_timeout(self, embed_video_logger, urllib_logger):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" as my_video %}
                {{ my_video.thumbnail }}
            {% endvideo %}
        """

        self.assertRenderedTemplate(template, "")

        urllib_logger.debug.assert_called_with(
            "Starting new HTTPS connection (%d): %s:%s", 1, "vimeo.com", 443
        )

        embed_video_logger.exception.assert_called_with(
            "Timeout reached during rendering embed video (`http://vimeo.com/72304002`)"
        )

    def test_relative_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" "80%x30%" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="80%" height="30%" '
            'src="https://player.vimeo.com/video/72304002" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_allow_spaces_in_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" "80% x 300" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="80%" height="300" '
            'src="https://player.vimeo.com/video/72304002" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_embed_with_query(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' query="rel=1&wmode=transparent" as ytb %}
               {{ ytb.url }}
           {% endvideo %}
        """

        output = self.render_template(template)
        self.assertUrlEqual(
            output, "https://www.youtube.com/embed/jsrRJyHBvzw?rel=1&wmode=transparent"
        )

    def test_direct_embed_with_query(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' query="rel=1&wmode=transparent" %}
        """

        output = self.render_template(template)

        # The order of query parameters in the URL might change between Python
        # versions. Compare the URL and the outer part separately.

        url_pattern = re.compile(r'http[^"]+')
        url = url_pattern.search(output).group(0)

        self.assertUrlEqual(
            url, "https://www.youtube.com/embed/jsrRJyHBvzw?rel=1&wmode=transparent"
        )

        output_without_url = url_pattern.sub("URL", output)

        self.assertEqual(
            output_without_url,
            '<iframe width="480" height="360" '
            'src="URL" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_set_options(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' "300x200" is_secure=True query="rel=1" %}
       """
        self.assertRenderedTemplate(
            template,
            '<iframe width="300" height="200" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?rel=1" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_size_as_variable(self):
        template = """
            {% load embed_video_tags %}
            {% with size="500x200" %}
                {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' size %}
            {% endwith %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="500" height="200" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )


class EmbedVideoNodeTestCase(TestCase):
    def setUp(self):
        self.parser = Mock()
        self.token = Mock(methods=["split_contents"])

    def test_repr(self):
        self.token.split_contents.return_value = (
            "video",
            "http://youtu.be/v/1234",
            "as",
            "myvideo",
        )
        self.parser.compile_filter.return_value = "some_url"

        node = VideoNode(self.parser, self.token)
        self.assertEqual(str(node), '<VideoNode "some_url">')

    def test_videonode_iter(self):
        out = ["a", "b", "c", "d"]

        class FooNode(VideoNode):
            nodelist_file = out

            def __init__(self):
                pass

        node = FooNode()
        self.assertEqual(out, [x for x in node])

    def test_get_backend_secure(self):
        class SecureRequest(RequestFactory):
            is_secure = lambda x: True

        context = {"request": SecureRequest()}
        backend = VideoNode.get_backend(
            "http://www.youtube.com/watch?v=jsrRJyHBvzw", context
        )
        self.assertTrue(backend.is_secure)

    def test_get_backend_insecure(self):
        class InsecureRequest(RequestFactory):
            is_secure = lambda x: False

        context = {"request": InsecureRequest()}
        backend = VideoNode.get_backend(
            "http://www.youtube.com/watch?v=jsrRJyHBvzw", context
        )
        self.assertFalse(backend.is_secure)



from json import dumps
from unittest import TestCase
from unittest.mock import patch

import requests
import requests_mock

from embed_video.backends import (
    SoundCloudBackend,
    VideoDoesntExistException,
    detect_backend,
)


class SoundCloudBackendTestCase(TestCase):
    urls = (
        (
            "https://soundcloud.com/community/soundcloud-case-study-wildlife",
            "82244706",
            "https://soundcloud.com/oembed?format=json&url=https%3A%2F%2Fsoundcloud.com%2Fcommunity%2Fsoundcloud-case-study-wildlife",
            dumps(
                {
                    "version": 1.0,
                    "type": "rich",
                    "provider_name": "SoundCloud",
                    "provider_url": "https://soundcloud.com",
                    "height": 400,
                    "width": "100%",
                    "title": "SoundCloud Case Study: Wildlife Control by SoundCloud Community",
                    "description": "Listen to how Wildlife Control makes the most of the SoundCloud platform, and it's API.",
                    "thumbnail_url": "https://i1.sndcdn.com/artworks-000042390403-ouou1g-t500x500.jpg",
                    "html": '<iframe width="100%" height="400" scrolling="no" frameborder="no" src="https://w.soundcloud.com/player/?visual=true&url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F82244706&show_artwork=true"></iframe>',
                    "author_name": "SoundCloud Community",
                    "author_url": "https://soundcloud.com/community",
                }
            ),
        ),
        (
            "https://soundcloud.com/matej-roman/jaromir-nohavica-karel-plihal-mikymauz",
            "7834701",
            "https://soundcloud.com/oembed?format=json&url=https%3A%2F%2Fsoundcloud.com%2Fmatej-roman%2Fjaromir-nohavica-karel-plihal-mikymauz",
            dumps(
                {
                    "version": 1.0,
                    "type": "rich",
                    "provider_name": "SoundCloud",
                    "provider_url": "https://soundcloud.com",
                    "height": 400,
                    "width": "100%",
                    "title": "Jaromír Nohavica, Karel Plíhal - Mikymauz by Matěj Roman",
                    "description": "",
                    "thumbnail_url": "https://soundcloud.com/images/fb_placeholder.png",
                    "html": '<iframe width="100%" height="400" scrolling="no" frameborder="no" src="https://w.soundcloud.com/player/?visual=true&url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F7834701&show_artwork=true"></iframe>',
                    "author_name": "Matěj Roman",
                    "author_url": "https://soundcloud.com/matej-roman",
                }
            ),
        ),
        (
            "https://soundcloud.com/beny97/sets/jaromir-nohavica-prazska",
            "960591",
            "https://soundcloud.com/oembed?format=json&url=https%3A%2F%2Fsoundcloud.com%2Fbeny97%2Fsets%2Fjaromir-nohavica-prazska",
            dumps(
                {
                    "version": 1.0,
                    "type": "rich",
                    "provider_name": "SoundCloud",
                    "provider_url": "https://soundcloud.com",
                    "height": 450,
                    "width": "100%",
                    "title": "Jaromir Nohavica - Prazska palena by beny97",
                    "description": "",
                    "thumbnail_url": "https://soundcloud.com/images/fb_placeholder.png",
                    "html": '<iframe width="100%" height="450" scrolling="no" frameborder="no" src="https://w.soundcloud.com/player/?visual=true&url=https%3A%2F%2Fapi.soundcloud.com%2Fplaylists%2F960591&show_artwork=true"></iframe>',
                    "author_name": "beny97",
                    "author_url": "https://soundcloud.com/beny97",
                }
            ),
        ),
        (
            "https://soundcloud.com/jet-silver/couleur-3-downtown-boogie-show-prove",
            "257485194",
            "https://soundcloud.com/oembed?format=json&url=https%3A%2F%2Fsoundcloud.com%2Fjet-silver%2Fcouleur-3-downtown-boogie-show-prove",
            dumps(
                {
                    "version": 1.0,
                    "type": "rich",
                    "provider_name": "SoundCloud",
                    "provider_url": "https://soundcloud.com",
                    "height": 400,
                    "width": "100%",
                    "title": "Couleur 3 - Downtown Boogie : Show & Prove by Jet Silver",
                    "description": "Show & Prove specially recorded for Downtown Boogie, the best hip-hop radio show in Switzerland, on Couleur 3 radio. Shout out to Vincz Lee, Jiggy Jones, Green Giant, Dynamike and Geos for having us one the ones and twos !",
                    "thumbnail_url": "https://i1.sndcdn.com/artworks-000156604478-47oo6y-t500x500.jpg",
                    "html": '<iframe width="100%" height="400" scrolling="no" frameborder="no" src="https://w.soundcloud.com/player/?visual=true&url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F257485194&show_artwork=true"></iframe>',
                    "author_name": "Jet Silver",
                    "author_url": "https://soundcloud.com/jet-silver",
                }
            ),
        ),
    )

    instance = SoundCloudBackend

    def setUp(self):
        class FooBackend(SoundCloudBackend):
            url = "foobar"

            def get_info(self):
                return {
                    "width": 123,
                    "height": 321,
                    "thumbnail_url": "xyz",
                    "html": '\u003Ciframe width="100%" height="400" '
                    'scrolling="no" frameborder="no" '
                    'src="{0}"\u003E\u003C/iframe\u003E'.format(self.url),
                }

        self.foo = FooBackend("abcd")

    def test_detect(self):
        for url in self.urls:
            with requests_mock.Mocker() as m:
                m.get(url[2], text=url[3])
                backend = detect_backend(url[0])
                self.assertIsInstance(backend, self.instance)

    def test_code(self):
        for url in self.urls:
            with requests_mock.Mocker() as m:
                m.get(url[2], text=url[3])
                backend = self.instance(url[0])
                self.assertEqual(backend.code, url[1])

    def test_width(self):
        self.assertEqual(self.foo.width, 123)

    def test_height(self):
        self.assertEqual(self.foo.height, 321)

    def test_get_thumbnail_url(self):
        self.assertEqual(self.foo.get_thumbnail_url(), "xyz")

    def test_get_url(self):
        self.assertEqual(self.foo.get_url(), self.foo.url)

    @patch("embed_video.backends.EMBED_VIDEO_TIMEOUT", 0.000001)
    def test_timeout_in_get_info(self):
        backend = SoundCloudBackend(
            "https://soundcloud.com/community/soundcloud-case-study-wildlife"
        )
        self.assertRaises(requests.Timeout, backend.get_info)

    def test_invalid_url(self):
        """Check if bug #21 is fixed."""
        backend = SoundCloudBackend("https://soundcloud.com/xyz/foo")
        self.assertRaises(VideoDoesntExistException, backend.get_info)



from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _

from embed_video.backends import (
    UnknownBackendException,
    UnknownIdException,
    VideoDoesntExistException,
    detect_backend,
)

__all__ = ("EmbedVideoField", "EmbedVideoFormField")


class EmbedVideoField(models.URLField):
    """
    Model field for embedded video. Descendant of
    :py:class:`django.db.models.URLField`.
    """

    def formfield(self, **kwargs):
        defaults = {"form_class": EmbedVideoFormField}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class EmbedVideoFormField(forms.URLField):
    """
    Form field for embeded video. Descendant of
    :py:class:`django.forms.URLField`
    """

    def validate(self, url):
        # if empty url is not allowed throws an exception
        super().validate(url)

        if not url:
            return

        try:
            backend = detect_backend(url)
            backend.get_code()
        except UnknownBackendException:
            raise forms.ValidationError(_("URL could not be recognized."))
        except UnknownIdException:
            raise forms.ValidationError(
                _("ID of this video could not be " "recognized.")
            )
        except VideoDoesntExistException:
            raise forms.ValidationError(_("This media not found on site."))
        return url



import logging
import re

import requests
from django.template import Library, Node, TemplateSyntaxError
from django.utils.encoding import smart_str
from django.utils.safestring import mark_safe

from embed_video.backends import (
    UnknownBackendException,
    VideoBackend,
    VideoDoesntExistException,
    detect_backend,
)

register = Library()

logger = logging.getLogger(__name__)


@register.tag("video")
class VideoNode(Node):
    """
    Template tag ``video``. It gives access to all
    :py:class:`~embed_video.backends.VideoBackend` variables.

    Usage (shortcut):

    .. code-block:: html+django

        {% video URL [SIZE] [key1=value1, key2=value2...] %}

    Or as a block:

    .. code-block:: html+django

        {% video URL [SIZE] [key1=value1, key2=value2...] as VAR %}
            ...
        {% endvideo %}

    Examples:

    .. code-block:: html+django

        {% video item.video %}
        {% video item.video "large" %}
        {% video item.video "340x200" %}
        {% video item.video "100% x 300" query="rel=0&wmode=opaque" %}

        {% video item.video is_secure=True as my_video %}
            URL: {{ my_video.url }}
            Thumbnail: {{ my_video.thumbnail }}
            Backend: {{ my_video.backend }}
        {% endvideo %}

    """

    error_msg = (
        "Syntax error. Expected: ``{% video URL "
        "[size] [key1=val1 key2=val2 ...] [as var] %}``"
    )
    default_size = "small"

    re_size = re.compile(r'[\'"]?(?P<width>\d+%?) *x *(?P<height>\d+%?)[\'"]?')
    re_option = re.compile(r"^(?P<key>[\w]+)=(?P<value>.+)$")

    def __init__(self, parser, token):
        """
        :param parser: Django template parser
        :type parser: django.template.base.Parser
        :param token: Django template token
        :type token: django.template.base.Token
        """
        self.parser = parser
        self.bits = list(token.split_contents())
        self.tag_name = str(self.pop_bit())
        self.url = self.pop_bit()

        if len(self.bits) > 1 and self.bits[-2] == "as":
            del self.bits[-2]
            self.variable_name = str(self.pop_bit(-1))
            self.nodelist_file = parser.parse(("end" + self.tag_name,))
            parser.delete_first_token()
        else:
            self.variable_name = None

        self.size = self.pop_bit() if self.bits and "=" not in self.bits[0] else None
        self.options = self.parse_options(self.bits)

    def pop_bit(self, index=0):
        return self.parser.compile_filter(self.bits.pop(index))

    def parse_options(self, bits):
        options = {}
        for bit in bits:
            parsed_bit = self.re_option.match(bit)
            key = smart_str(parsed_bit.group("key"))
            value = self.parser.compile_filter(parsed_bit.group("value"))
            options[key] = value
        return options

    def render(self, context):
        """
        Returns generated HTML.

        :param context: Django template RequestContext
        :type context: django.template.RequestContext
        :return: Rendered HTML with embed video.
        :rtype: django.utils.safestring.SafeText | str
        """
        url = self.url.resolve(context)
        size = self.size.resolve(context) if self.size else None
        options = self.resolve_options(context)

        try:
            if not self.variable_name:
                return self.embed(url, size, context=context, **options)
            backend = self.get_backend(url, context=context, **options)
            return self.render_block(context, backend)
        except requests.Timeout:
            logger.exception(
                "Timeout reached during rendering embed video (`{0}`)".format(url)
            )
        except UnknownBackendException:
            logger.warning("Backend wasn't recognised (`{0}`)".format(url))
        except VideoDoesntExistException:
            logger.warning("Attempt to render not existing video (`{0}`)".format(url))

        return ""

    def resolve_options(self, context):
        """
        :param context: Django template RequestContext
        :type context: django.template.RequestContext
        """
        options = {}
        for key in self.options:
            value = self.options[key]
            options[key] = value.resolve(context)
        return options

    def render_block(self, context, backend):
        """
        :param context: Django template RequestContext
        :type context: django.template.RequestContext
        :param backend: Given instance inherited from VideoBackend
        :type backend: VideoBackend
        :rtype: django.utils.safestring.SafeText
        """
        context.push()
        context[self.variable_name] = backend
        output = self.nodelist_file.render(context)
        context.pop()
        return output

    @staticmethod
    def get_backend(backend_or_url, context=None, **options):
        """
        Returns instance of VideoBackend. If context is passed to the method
        and request is secure, than the is_secure mark is set to backend.

        A string or VideoBackend instance can be passed to the method.

        :param backend: Given instance inherited from VideoBackend or url
        :type backend_or_url: VideoBackend | str
        :param context: Django template RequestContext
        :type context: django.template.RequestContext | None
        :rtype: VideoBackend
        """

        backend = (
            backend_or_url
            if isinstance(backend_or_url, VideoBackend)
            else detect_backend(str(backend_or_url))
        )

        if context and "request" in context:
            backend.is_secure = context["request"].is_secure()
        if options:
            backend.set_options(options)

        return backend

    @classmethod
    def embed(cls, url, size, context=None, **options):
        """
        Direct render of embed video.

        :param url: URL to embed video
        :type url: str
        :param size: Size of rendered block
        :type size: str
        :param context: Django template RequestContext
        :type context: django.template.RequestContext | None
        """
        backend = cls.get_backend(url, context=context, **options)
        width, height = cls.get_size(size)
        return mark_safe(backend.get_embed_code(width=width, height=height))

    @classmethod
    def get_size(cls, value):
        """
        Predefined sizes:

        ======== ======== =========
        size     width    height
        ======== ======== =========
        tiny     420      315
        small    480      360
        medium   640      480
        large    960      720
        huge     1280     960
        ======== ======== =========

        You can also use custom size - in format ``WIDTHxHEIGHT``
        (eg. ``500x400``).

        :type value: str

        :return: Returns tuple with (width, height) values.
        :rtype: tuple[int, int]
        """
        sizes = {
            "tiny": (420, 315),
            "small": (480, 360),
            "medium": (640, 480),
            "large": (960, 720),
            "huge": (1280, 960),
        }

        value = value or cls.default_size
        if value in sizes:
            return sizes[value]

        try:
            size = cls.re_size.match(value)
            return size.group("width"), size.group("height")
        except AttributeError:
            raise TemplateSyntaxError(
                "Incorrect size.\nPossible format is WIDTHxHEIGHT or using "
                "predefined size ({sizes}).".format(sizes=", ".join(sizes.keys()))
            )

    def __iter__(self):
        for node in self.nodelist_file:
            yield node

    def __repr__(self):
        return '<VideoNode "%s">' % self.url



from django.db import models
from django.urls import reverse

from embed_video.fields import EmbedVideoField


class Post(models.Model):
    title = models.CharField(max_length=50)
    video = EmbedVideoField(verbose_name="My video", help_text="This is a help text")

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("posts:detail", kwargs={"pk": self.pk})

    class Meta:
        ordering = ["pk"]



from unittest import TestCase
from unittest.mock import patch

import requests

from embed_video.backends import VideoDoesntExistException, VimeoBackend, detect_backend


class VimeoBackendTestCase(TestCase):
    urls = (
        ("http://vimeo.com/72304002", "72304002"),
        ("https://vimeo.com/72304002", "72304002"),
        ("http://www.vimeo.com/72304002", "72304002"),
        ("https://www.vimeo.com/72304002", "72304002"),
        ("http://player.vimeo.com/video/72304002", "72304002"),
        ("https://player.vimeo.com/video/72304002", "72304002"),
        ("http://www.vimeo.com/channels/staffpick/72304002", "72304002"),
        ("https://www.vimeo.com/channels/staffpick/72304002", "72304002"),
        ("https://vimeo.com/exampleusername/review/72304002/a1b2c3d4", "72304002"),
        ("https://vimeo.com/manage/72304002/general", "72304002"),
    )

    instance = VimeoBackend

    def test_detect(self):
        for url in self.urls:
            backend = detect_backend(url[0])
            self.assertIsInstance(backend, self.instance)

    def test_code(self):
        for url in self.urls:
            backend = self.instance(url[0])
            self.assertEqual(backend.code, url[1])

    def test_vimeo_get_info_exception(self):
        with self.assertRaises(VideoDoesntExistException):
            backend = VimeoBackend("https://vimeo.com/123")
            backend.get_info()

    def test_get_thumbnail_url(self):
        backend = VimeoBackend("https://vimeo.com/72304002")
        self.assertEqual(
            backend.get_thumbnail_url(),
            "https://i.vimeocdn.com/video/446150690-9621b882540b53788eaa36ef8e303d4e06fc40af3d27918b7f561bb44ed971dc-d_640",
        )

    @patch("embed_video.backends.EMBED_VIDEO_TIMEOUT", 0.000001)
    def test_timeout_in_get_info(self):
        backend = VimeoBackend("https://vimeo.com/72304002")
        self.assertRaises(requests.Timeout, backend.get_info)



from django.conf import settings

EMBED_VIDEO_BACKENDS = getattr(
    settings,
    "EMBED_VIDEO_BACKENDS",
    (
        "embed_video.backends.YoutubeBackend",
        "embed_video.backends.VimeoBackend",
        "embed_video.backends.SoundCloudBackend",
    ),
)
""" :type: tuple[str] """

EMBED_VIDEO_TIMEOUT = getattr(settings, "EMBED_VIDEO_TIMEOUT", 10)
""" :type: int """

EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY = getattr(
    settings, "EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY", "wmode=opaque"
)
""" :type: django.db.models.QuerySet | str """

EMBED_VIDEO_YOUTUBE_CHECK_THUMBNAIL = getattr(
    settings, "EMBED_VIDEO_YOUTUBE_CHECK_THUMBNAIL", True
)
""" :type: bool """



from unittest import TestCase

from embed_video.backends import UnknownBackendException, VideoBackend, detect_backend


class VideoBackendTestCase(TestCase):
    unknown_backend_urls = (
        "http://myurl.com/?video=http://www.youtube.com/watch?v=jsrRJyHBvzw",
        "http://myurl.com/?video=www.youtube.com/watch?v=jsrRJyHBvzw",
        "http://youtube.com.myurl.com/watch?v=jsrRJyHBvzw",
        "http://vimeo.com.myurl.com/72304002",
    )

    def test_detect_bad_urls(self):
        for url in self.unknown_backend_urls:
            self.assertRaises(UnknownBackendException, detect_backend, url)

    def test_not_implemented_get_info(self):
        backend = VideoBackend("https://www.example.com")
        self.assertRaises(NotImplementedError, backend.get_info)



import re
from unittest import TestCase

from embed_video.backends import VideoBackend, detect_backend


class CustomBackend(VideoBackend):
    re_detect = re.compile(r"http://myvideo\.com/[0-9]+")
    re_code = re.compile(r"http://myvideo\.com/(?P<code>[0-9]+)")

    pattern_url = "{protocol}://play.myvideo.com/c/{code}/"
    pattern_thumbnail_url = "{protocol}://thumb.myvideo.com/c/{code}/"


class CustomBackendTestCase(TestCase):
    def setUp(self):
        self.backend = detect_backend("http://myvideo.com/1530")

    def test_detect_backend(self):
        self.assertIsInstance(self.backend, CustomBackend)

    def test_code(self):
        self.assertEqual(self.backend.code, "1530")

    def test_url(self):
        self.assertEqual(self.backend.get_url(), "http://play.myvideo.com/c/1530/")

    def test_url_https(self):
        self.backend.is_secure = True
        self.assertEqual(self.backend.get_url(), "https://play.myvideo.com/c/1530/")

    def test_thumbnail(self):
        self.assertEqual(
            self.backend.get_thumbnail_url(), "http://thumb.myvideo.com/c/1530/"
        )



from unittest import TestCase
from unittest.mock import patch

from django.forms import ValidationError

from embed_video.backends import (
    UnknownBackendException,
    UnknownIdException,
    YoutubeBackend,
)
from embed_video.fields import EmbedVideoField, EmbedVideoFormField


class EmbedVideoFieldTestCase(TestCase):
    def setUp(self):
        self.field = EmbedVideoField()

    def test_formfield_form_class(self):
        self.assertIsInstance(self.field.formfield(), EmbedVideoFormField)


class EmbedVideoFormFieldTestCase(TestCase):
    def setUp(self):
        self.formfield = EmbedVideoFormField()

    def test_validation_unknown_backend(self):
        with patch("embed_video.fields.detect_backend") as mock_detect_backend:
            mock_detect_backend.return_value = True
            mock_detect_backend.side_effect = UnknownBackendException
            self.assertRaises(
                ValidationError, self.formfield.validate, ("http://youtube.com/v/123/",)
            )

    def test_validation_unknown_id(self):
        with patch("embed_video.fields.detect_backend") as mock_detect_backend:
            mock_detect_backend.return_value = True
            mock_detect_backend.side_effect = UnknownIdException
            self.assertRaises(
                ValidationError, self.formfield.validate, ("http://youtube.com/v/123/",)
            )

    def test_validation_correct(self):
        url = "http://www.youtube.com/watch?v=gauN0gzxTcU"
        with patch("embed_video.fields.detect_backend") as mock_detect_backend:
            mock_detect_backend.return_value = YoutubeBackend(url)
            self.assertEqual(url, self.formfield.validate(url))

    def test_validation_unknown_code(self):
        url = "http://www.youtube.com/edit?abcd=abcd"
        self.assertRaises(ValidationError, self.formfield.validate, url)

    def test_validation_super(self):
        self.assertRaises(ValidationError, self.formfield.validate, "")

    def test_validation_allowed_empty(self):
        formfield = EmbedVideoFormField(required=False)
        self.assertIsNone(formfield.validate(""))



from django.contrib import admin

from embed_video.admin import AdminVideoMixin

from .models import Post


class PostAdmin(AdminVideoMixin, admin.ModelAdmin):
    pass


admin.site.register(Post, PostAdmin)
